# $Id: SpellPolish.pm,v 1.13 2002/08/06 15:30:07 radek Exp $
package Number::SpellPolish;

require 5.003;
use strict;

use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK $VERSION);

require Exporter;
@ISA = qw(Exporter);

%EXPORT_TAGS = ( 'all' => [ qw( &na_zlote &na_grosze &slownie ) ] );
@EXPORT_OK = ( qw(  ), @{ $EXPORT_TAGS{'all'} } );

$VERSION = '0.7';

#use vars qw($Only7bit);


my @setki = (undef, qw/sto       dwiecie   trzysta   czterysta  piset
                       szeset  siedemset  osiemset  dziewiset/);

my @liczby = (undef, qw/jedenacie  dwanacie     trzynacie   czternacie  pitnacie
                        szesnacie  siedemnacie  osiemnacie  dziewitnacie/);

my @dziesiatki = (undef, qw/dziesi       dwadziecia   trzydzieci
                            czterdzieci   pidziesit  szedziesit
                            siedemdziesit osiemdziesit dziewidziesit/);

my @jednosci = (undef, qw/jeden dwa trzy cztery pi sze siedem osiem dziewi/);


#
# nazwy duych liczb.  rdo:
# <http://star.wckp.lodz.pl/server/prace/fiz_gim/inne/symbole/przed.htm>
#
my %bignum = (  # 10 ** key => [ spelling ]                 # USA
      3 => [ qw/tysic      tysice      tysicy/ ],        
      6 => [ qw/milion      miliony      milionw/ ],       
      9 => [ qw/miliard     miliardy     miliardw/ ],      # bilion
     12 => [ qw/bilion      biliony      bilionw/ ],       # trylion
     15 => [ qw/biliard     biliardy     biliardw/ ],      # kwadrylion
     18 => [ qw/trylion     tryliony     trylionw/ ],      # kwintylion
     21 => [ qw/tryliard    tryliardy    tryliardw/ ],     # sekstylion
     24 => [ qw/kwadrylion  kwadryliony  kwadrylionw/ ],   # septylion
     27 => [ qw/kwadryliard kwadryliardy kwadryliardw/ ],  # oktylion
     30 => [ qw/kwintylion  kwintyliony  kwintylionw/ ],   # nonylion
     33 => [ qw/kwintyliard kwintyliardy kwintyliardw/ ],  # decylion
     36 => [ qw/sekstylion  sekstyliony  sekstylionw/ ],
     39 => [ qw/sekstyliard sekstyliardy sekstyliardw/ ],
     42 => [ qw/septylion   septyliony   septylionw/ ],
     45 => [ qw/septyliard  septyliardy  septyliardw/ ],
     48 => [ qw/oktylion    oktyliony    oktylionw/ ],
     51 => [ qw/oktyliard   oktyliardy   oktyliardw/ ],
     54 => [ qw/nonylion    nonyliony    nonylionw/ ],
     57 => [ qw/nonyliard   nonyliardy   nonyliardw/ ],
     60 => [ qw/decylion    decyliony    decylionw/ ],
#   ...
#   100 => [ qw/gogol gogole gogoli/ ],
#   600 => [ qw/centylion centyliony centylionw/ ],
);


my $max = (sort { $b <=> $a } keys %bignum)[0];
$max = 10**$max;


sub get_args {  # parse arguments passed to functions
	my($num, %args);
	$num = shift;
	$num ||= 0;
	return $num, %args unless @_;
	my @ORDER = qw/no_8_bit/;
	if    (ref $_[0] eq 'ARRAY') { @args{@ORDER} = @{$_[0]} }
	elsif (ref $_[0] eq 'HASH')  { %args = %{$_[0]} } 
	else                         { @args{@ORDER} = @_ }
	return $num, %args;
}

sub na_zlote {  # spell a number in zlote
	my($input, %args) = get_args(@_);

	if ($input !~ /\d+/) {
		warn "brak liczby!";
		return undef;
	}

#   return $input;
	my($zl, $gr) = $input =~ /(\d+)(?:\.(\d{1,2}))?/;
#   (my $num) = $input =~ /(\d+(?:\.\d+)?(?:e\+?\d+(?:\+\d+)?)?)/;
	$zl ||= 0;
	$gr .= 0 if $gr && length $gr < 2;

	my $ret = slownie($zl);

	my $jed = $zl % 1000;
	if ($jed == 1) {
		$ret .= " zoty "
	} elsif ($jed % 10 >= 2 && $jed % 10 <= 4 && ( $jed % 100 < 10 || $jed %  100 > 20)) {
		$ret .= " zote "
	} else {
		$ret .= " zotych "
	}  

	$ret .= 'i ' . na_grosze($gr) if $gr;

	$ret =~ y/󱶳ӡ/eoaslzzcnEOASLZZCN/ if $args{no_8_bit};
	$ret =~ y/ / /s;
	$ret =~ s/^\s|\s$//g;
	$ret;
}

sub na_grosze {  # spell a number in grosze
	my($input, %args) = get_args(@_);

	if ($input !~ /\d+/) {
		warn "brak liczby!";
		return undef;
	}

	(my $gr) = $input =~ /(\d+)/;
	$gr ||= 0;
	my $ret;

	return "zero groszy" if $gr == 0;

	$ret .= slownie($gr);

	if ($gr && $gr == 1) {
		$ret .= " grosz"
	} elsif ($gr && $gr % 10 >= 2 && $gr % 10 <= 4 && ( $gr % 100 < 10 || $gr % 100 >= 20)) {
		$ret .= " grosze"
	} elsif ($gr) {
		$ret .= " groszy"
	}
	
	$ret =~ y/󱶳ӡ/eoaslzzcnEOASLZZCN/ if $args{no_8_bit};
	$ret =~ y/ / /s;
	$ret =~ s/^\s|\s$//g;
	$ret;
}

sub slownie {
	my($num, %args) = get_args(@_);

	$num =~ s/^[^\d]*(\d+).*$/$1/s;
	if ($num !~ /\d+/) {
		warn "brak liczby!";
		return undef;
	}

	return "zero" if $num == 0;

	# dzielimy liczb na porcje po trzy cyfry, od koca
	my @ret = map { scalar(reverse)+0 } grep length, split /(\d{3})/, scalar reverse $num;

	$num = shift @ret;  # pierwszy element

	for (0 .. $#ret) {
		my $p = ($_ + 1) * 3;
		my $ret;

		$ret = i2pl($ret[$_]) if $ret[$_];

		if ($bignum{$p} && $ret[$_] > 0) {
			if ($ret[$_] == 1) { $ret .= "$bignum{$p}[0] " }
			elsif ($ret[$_] % 10 >= 2
				&& $ret[$_] % 10 <= 4
				&& ($ret[$_] % 100 < 10 || $ret[$_] % 100 > 20))
			{
				$ret .= "$bignum{$p}[1] ";
			}
			elsif ($ret[$_] < 1000) { $ret .= "$bignum{$p}[2] " }
		}
		$ret ||= " ";
#		$ret =~ s/^\s+|\s+$//;
		$ret[$_] = $ret;
	}

	unshift @ret, i2pl($num) if $num;

	my $ret = join " ", grep $_, reverse @ret;
	$ret =~ y/󱶳ӡ/eoaslzzcnEOASLZZCN/ if $args{no_8_bit};
	$ret =~ y/ / /s;
	$ret =~ s/^\s|\s$//g;
	$ret;
}



sub i2pl {
	my $num = shift;
	my $ret;

	if ($num == 0) {
		$ret = "zero "
	} else {
		my $set = sprintf('%d', $num / 100);
		$ret .= "$setki[$set] " if $set;
		my $jedn = $num % 100;
		if ($jedn > 10 && $jedn < 20) {
			my $liczb = $jedn - 10;
			$ret .= "$liczby[$liczb] ";
		} else {
			my $dzies = ($num / 10) % 10;
			$ret .= "$dziesiatki[$dzies] " if $dzies;
			$jedn = $num % 10;
			$ret .= "$jednosci[$jedn] " if $jedn;
		}
	}
	$ret;
}


1;


=head1 NAME

Number::SpellPolish - Perl extension for spelling out numbers in Polish

=head1 SYNOPSIS

  use Number::SpellPolish qw(slownie na_zlote na_grosze); # or qw(:all) 
  $spelled_number = slownie($number);
  $spelled_price  = na_zlote($price);
  
  # without polish locale characters (ISO-8859-2):
  $spelled_number = slownie($number, 'no_8_bit');

=head1 DESCRIPTION

Number::SpellPolish provides functionality for spelling out numbers and
prices in Polish language.  If you don't speak polish, you probably won't
need it.  I don't really know why am I writing this in English... ;->

This module I<is not> a subclass of Number::Spell.

=head1 FUNCTIONS

=over

=item 
I<slownie>(NUMBER, OPTIONS)

Returns spelled NUMBER.

=item 
I<na_zlote>(NUMBER, OPTIONS)

Treats NUMBER as price in zlotys.

=item 
I<na_grosze>(NUMBER, OPTIONS)

Treats NUMBER as price in grosze.

=back

These functions return string on success and undef on failure.

=head2 NUMBER

Should be a string containing integer or float.
See L<BUGS> for hints about large numbers.

Number::SpellPolish does not recognize scientific notation since version
0.7: numbers are treated as strings internally.  Contact me if you
need it; I'd consider adding it (if you'll provide a converter to the
traditional version, it will be much faster ;-).

=head2 OPTIONS

May be passed as LIST, ARRAY reference or HASH reference.

Examples:

    $num = slownie(10, no_8_bit);
    $num = slownie(10, [ no_8_bit ]);
    $num = slownie(10, { no_8_bit => 'true' });

Valid options are:

=over

=item 
I<no_8_bit>

Strips polish locale characters from numbers.  You have to have ISO-8859-2
fonts to see them.

=back

=head1 BUGS

You have to pass large (over 1e15) numbers as strings: slownie("12345")),
not numbers: slownie(12345).

I'm not sure if large numbers are spelled correctly.

Code badly needs cleanup.  It would be nice to provide a consistent
interface for other languages by making Number::Spell a base class;
after then, this module should became it's subclass.

As far as I know, everything works OK (for not too large numbers).
If you'll find something, drop me a note.

=head1 AUTHOR

Radoslaw Zielinski E<lt>radek@karnet.plE<gt>.

This module is based on C code written by Piotr Piatkowski E<lt>kompas@ceti.com.plE<gt>,
translated to Perl by Lukasz Zalubski E<lt>chopin@tau.ceti.com.plE<gt> (distributed as
'slownie.pl' library).  I had fixed it, rebuilt and packed into this module.

=head1 LICENSE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

=head1 SEE ALSO

perl(1).

=cut
